<?php

add_action('bricks/form/custom_action', 'oopspam_spam_check', 10, 1);

function oopspam_spam_check($form)
{
    $fields = $form->get_fields();
    $formId = $fields['formId'];
    $settings = $form->get_settings();

    $options = get_option('oopspamantispam_settings');

    if (!empty($options['oopspam_api_key']) && !empty($options['oopspam_is_br_activated'])) {

        // Get email and content field Ids
        $msgFieldId = "";
        $emailFieldId = "";
        foreach ($settings['fields'] as $field) {
            if ($field["type"] == "email") {
                $emailFieldId = $field["id"];
            }
            if ($field["type"] == "textarea") {
                $msgFieldId = $field["id"];
            }
        }

        // In Bricks, every form field starts with "form-field-[ID]"
        $message = "";
        $email = "";

        // Capture the content
        if (!empty($msgFieldId)) {

            if (isset($options['oopspam_br_content_field']) && $options['oopspam_br_content_field']) {
                $customContentField = sanitize_text_field(trim($options['oopspam_br_content_field']));
                $message = $fields["form-field-" . $customContentField];
            } else {
                $message = sanitize_text_field($fields["form-field-" . $msgFieldId]);
            }

        }

        // Capture the email
        if (!empty($emailFieldId)) {
            $email = sanitize_email($fields["form-field-" . $emailFieldId]);
        }

        $raw_entry = json_encode($fields);
        $userIP = "";

        if (!isset($options['oopspam_is_check_for_ip']) || $options['oopspam_is_check_for_ip'] != true) {
            $userIP = oopspamantispam_get_ip();
        }

        $escapedMsg = sanitize_textarea_field($message);
        $detectionResult = oopspamantispam_call_OOPSpam($escapedMsg, $userIP, $email, true);

        if (!isset($detectionResult["isItHam"])) {
            return;
        }

        $frmEntry = [
            "Score" => $detectionResult["Score"],
            "Message" => $escapedMsg,
            "IP" => $userIP,
            "Email" => $email,
            "RawEntry" => $raw_entry,
            "FormId" => $formId,
        ];

        if (!$detectionResult["isItHam"]) {
            // It's spam, store the submission and show error
            oopspam_store_spam_submission($frmEntry);

            $error_to_show = $options['oopspam_br_spam_message'];
            // Show error
            $form->set_result([
                'action' => 'oopspam_spam_check',
                'type' => 'danger',
                'message' => esc_html__($error_to_show, 'bricks'),
            ]);
        } else {
            // It's ham
            oopspam_store_ham_submission($frmEntry);
        }

    }

    return;
}
