<?php

add_action('elementor_pro/forms/validation', 'oopspamantispam_el_pre_submission', 10, 2);

function oopspamantispam_el_pre_submission($record, $ajax_handler)
{

    $options = get_option('oopspamantispam_settings');

    // get password types and later remove them from $raw_entry so that it's not stored locally
    $password_fields = $record->get_field([
        'type' => 'password',
    ]);

    $text_fields = $record->get_field([
        'type' => 'textarea',
    ]);

    $email_fields = $record->get_field([
        'type' => 'email',
    ]);

    $field = current($text_fields);
    $email_field = current($email_fields);

    if (!empty($options['oopspam_api_key']) && !empty($options['oopspam_is_el_activated'])) {

        // Capture the content
        $message = "";
        if (is_array($field) && !empty($field)) {
            // by default, the capture textarea field
            $message = $field['value'];

            // unless it's custom field ID is set by the user
            if (isset($options['oopspam_el_content_field']) && $options['oopspam_el_content_field']) {
                $customContentField = sanitize_text_field(trim($options['oopspam_el_content_field']));
                $message = $record->get('sent_data')[$customContentField];
            }
        }

        // Capture the email
        $email = "";
        if (is_array($email_field) && !empty($email_field)) {
            $email = $email_field['value'];
        }


        $raw_entry = $record->get('sent_data');

        // Remove any password fields
        if($password_fields) {
            foreach ($password_fields as $field => $field_val) {
                unset($raw_entry[$field]);
            }
        }
        $raw_entry = json_encode($raw_entry);


        $form_id = $record->get('form_settings');
        $userIP = "";

        if (!isset($options['oopspam_is_check_for_ip']) || $options['oopspam_is_check_for_ip'] != true) {
            $userIP = \ElementorPro\Core\Utils::get_client_ip();
        }

        $escapedMsg = sanitize_textarea_field($message);
        $detectionResult = oopspamantispam_call_OOPSpam($escapedMsg, $userIP, $email, true);

        if (!isset($detectionResult["isItHam"])) {
            return;
        }

        $frmEntry = [
            "Score" => $detectionResult["Score"],
            "Message" => $escapedMsg,
            "IP" => $userIP,
            "Email" => $email,
            "RawEntry" => $raw_entry,
            "FormId" => $form_id["form_name"],
        ];

        if (!$detectionResult["isItHam"]) {
            // It's spam, store the submission and show error
            oopspam_store_spam_submission($frmEntry);

            $field_id = "";
            // Content field isn't available. Capture first item's ID from the array to show the error.
            if (empty($field['id'])) {
                $field_id = array_keys($record->get('sent_data'))[0];
            } else {
                $field_id = $field['id'];
            }

            $error_to_show = $options['oopspam_el_spam_message'];
            $ajax_handler->add_error($field_id, $error_to_show);
        } else {
            // It's ham
            oopspam_store_ham_submission($frmEntry);
        }

    }

    return;
}
