<?php

add_action('fluentform_before_insert_submission', 'oopspamantispam_ff_pre_submission', 20, 3);

function oopspamantispam_ff_pre_submission($insertData, $data, $form)
{

    $options = get_option('oopspamantispam_settings');
    $fields = json_decode($form->form_fields, true);
    $email = "";
    $message = "";

    // Default "textarea" field name in Fluent Forms is "description"
    $nameOfTextareaField = "description";

    // unless it's custom field ID is set by the user
    if (isset($options['oopspam_ff_content_field']) && $options['oopspam_ff_content_field']) {
        $nameOfTextareaField = sanitize_text_field(trim($options['oopspam_ff_content_field']));
    } else {
        // Attempt to find custom textarea field name
        foreach ($fields["fields"] as $field) {
            if ($field["element"] == "textarea") {
                $nameOfTextareaField = $field["attributes"]["name"];
            }
        }
    }

    // Capture the email
    if (isset($data["email"])) {
        $email = sanitize_email($data["email"]);
    }

    // Capture the content
    if (isset($data[$nameOfTextareaField])) {
        $message = $data[$nameOfTextareaField];
    }

    if (!empty($options['oopspam_api_key']) && !empty($options['oopspam_is_ff_activated'])) {

        $userIP = "";
        if (!isset($options['oopspam_is_check_for_ip']) || $options['oopspam_is_check_for_ip'] != true) {
            $userIP = $insertData["ip"];
        }

        $escapedMsg = sanitize_textarea_field($message);
        $detectionResult = oopspamantispam_call_OOPSpam($escapedMsg, $userIP, $email, true);
        $raw_entry = json_encode($data);

        if (!isset($detectionResult["isItHam"])) {
            return;
        }

        $frmEntry = [
            "Score" => $detectionResult["Score"],
            "Message" => $escapedMsg,
            "IP" => $userIP,
            "Email" => $email,
            "RawEntry" => $raw_entry,
            "FormId" => $insertData["form_id"],
        ];

        if (!$detectionResult["isItHam"]) {

            // It's spam, store the submission and show error
            oopspam_store_spam_submission($frmEntry);

            $error_to_show = $options['oopspam_ff_spam_message'];
            wp_send_json(['errors' => [
                'restricted' => [
                    $error_to_show,
                ],
            ]], 422);
        } else {
            // It's ham
            oopspam_store_ham_submission($frmEntry);
        }
    }

    return;

}
