<?php
add_filter('ninja_forms_submit_data', 'oopspamantispam_forms_after_submission');

function oopspamantispam_forms_after_submission($form_data)
{

    $options = get_option('oopspamantispam_settings');

    if (!empty($options['oopspam_api_key']) && !empty($options['oopspam_is_nj_activated'])) {
        $message = "";
        $field_id = "";
        $email = "";
        // Default Field Key (aka ID) starts with "textarea"
        $keyToLook = "textarea";
        if (isset($options['oopspam_nj_content_field']) && $options['oopspam_nj_content_field']) {
            $keyToLook = sanitize_text_field(trim($options['oopspam_nj_content_field']));
        }

        foreach ($form_data['fields'] as $field) { // Field settings, including the field key and value.
            // Capture the email field
            if (strpos($field['key'], "email") !== false && empty($email)) {
                $email = sanitize_email($field['value']);
                if (!empty($message))
                break;
            }
            // Look for textarea field only
            if (strpos($field['key'], $keyToLook) !== false) {
                $message = $field['value'];
                $field_id = $field['id'];
                if (!empty($email))
                break;
            }
        }

        $userIP = "";
        if (!isset($options['oopspam_is_check_for_ip']) || $options['oopspam_is_check_for_ip'] != true) {
            $userIP = oopspamantispam_get_ip();
        }
        $escapedMsg = sanitize_textarea_field($message);
        $raw_entry = json_encode($form_data['fields']);
        $detectionResult = oopspamantispam_call_OOPSpam($escapedMsg, $userIP, $email, true);
        if (!isset($detectionResult["isItHam"])) {
            return $form_data;
        }
        $frmEntry = [
            "Score" => $detectionResult["Score"],
            "Message" => $escapedMsg,
            "IP" => $userIP,
            "Email" => $email,
            "RawEntry" => $raw_entry,
            "FormId" => $form_data["id"],
        ];

        if (!$detectionResult["isItHam"]) {

            // It's spam, store the submission and show error
            oopspam_store_spam_submission($frmEntry);

            $error_to_show = $options['oopspam_nj_spam_message'];

            // Content field isn't available. Capture first item's ID from the array to show the error.
            if (empty($field_id)) {
                $field_id = array_values($form_data['fields'])[0]['id'];
            }
            $errors = [
                'fields' => [
                    $field_id => __($error_to_show, 'oopspam'),
                ],
            ];

            $response = [
                'errors' => $errors,
            ];
            return $response;
        } else {
            // It's ham
            oopspam_store_ham_submission($frmEntry);
        }

    }
    return $form_data;
}
