<?php

add_action('wpforms_process', 'oopspamantispam_wpf_pre_submission', 10, 4);

function oopspamantispam_wpf_pre_submission($fields, $entry, $form_data)
{

    $options = get_option('oopspamantispam_settings');
    $message = "";
    $email = "";

    if (empty($fields)) {
        return;
    }

    // Check if a custom Field ID is set
    if (isset($options['oopspam_wpf_content_field']) && $options['oopspam_wpf_content_field']) {
        $nameOfTextareaField = sanitize_text_field(trim($options['oopspam_wpf_content_field']));
        foreach ($fields as $field) {
            if ($field["id"] == $nameOfTextareaField) {
                $message = $field["value"];
            }
            if ($field["type"] == "email") {
                $email = sanitize_email($field["value"]);
            }
        }
    } else {
        // Attempt to find textarea field name
        foreach ($fields as $field) {
            if ($field["type"] == "textarea") {
                $message = $field["value"];
            }
            if ($field["type"] == "email") {
                $email = sanitize_email($field["value"]);
            }
        }
    }

    if (!empty($options['oopspam_api_key']) && !empty($options['oopspam_is_wpf_activated'])) {

        $userIP = "";
        if (!isset($options['oopspam_is_check_for_ip']) || $options['oopspam_is_check_for_ip'] != true) {
            $userIP = wpforms_get_ip();
        }
        $escapedMsg = sanitize_textarea_field($message);
        $raw_entry = json_encode($fields);
        $detectionResult = oopspamantispam_call_OOPSpam($escapedMsg, $userIP, $email, true);
        if (!isset($detectionResult["isItHam"])) {
            return;
        }
        $frmEntry = [
            "Score" => $detectionResult["Score"],
            "Message" => $escapedMsg,
            "IP" => $userIP,
            "Email" => $email,
            "RawEntry" => $raw_entry,
            "FormId" => $form_data['id'],
        ];

        if (!$detectionResult["isItHam"]) {

            // It's spam, store the submission and show error
            oopspam_store_spam_submission($frmEntry);
            $error_to_show = $options['oopspam_wpf_spam_message'];
            wpforms()->process->errors[$form_data['id']]['footer'] = esc_html__($error_to_show, 'oopspam-antispam');
        } else {
            // It's ham
            oopspam_store_ham_submission($frmEntry);
        }

    }

    return;
}
