<?php
add_filter('oopspam_check_for_spam_tag', 'oopspamantispam_ws_pre_submission', 10, 2);

// Filter function
function oopspamantispam_ws_pre_submission($form, $submit)
{

    $options = get_option('oopspamantispam_settings');
    // Perform your spam check here
    $spam_level = 0;

    if (!empty($options['oopspam_api_key']) && !empty($options['oopspam_is_ws_activated'])) {

        $form_id = $submit->form_id;
        $field_id = "";
        $message = "";
        $email = "";
        $userIP = "";
        $raw_entry = "";

        // Check if a custom content field is set
        if (isset($options['oopspam_ws_content_field']) && $options['oopspam_ws_content_field']) {
            $field_id = 'field_' . sanitize_text_field(trim($options['oopspam_ws_content_field']));
            // Capture the content field based on custom Field ID
            $message = $submit->meta[$field_id]['value'];
        } else {
            // No custom Field ID set, capture the first matching textarea field
            foreach ($submit->meta as $field) {
                if (isset($field["type"]) && $field["type"] === "textarea") {
                    $message = $field["value"];
                    break;
                }
            }

        }
        // Clean message field
        $escapedMsg = sanitize_textarea_field($message);

        // Capture user's IP if allowed
        if (!isset($options['oopspam_is_check_for_ip']) || $options['oopspam_is_check_for_ip'] != true) {
            $userIP = oopspamantispam_get_ip();
        }

        // Capture email field
        foreach ($submit->meta as $field) {
            if (isset($field["type"]) && $field["type"] === "email") {
                $email = $field["value"];
                break;
            }
        }

        // Capture raw entry
        $raw_entry = json_encode($submit->meta);

        $detectionResult = oopspamantispam_call_OOPSpam($escapedMsg, $userIP, $email, true);
        if (!isset($detectionResult["isItHam"])) {
            return $form_data;
        }
        $frmEntry = [
            "Score" => $detectionResult["Score"],
            "Message" => $escapedMsg,
            "IP" => $userIP,
            "Email" => $email,
            "RawEntry" => $raw_entry,
            "FormId" => $form_id,
        ];

        if (!$detectionResult["isItHam"]) {

            // It's spam, store the submission in Form Spam Entries
            oopspam_store_spam_submission($frmEntry);

            // It's spam, give high score
            $spam_level = 100;
            return $spam_level;
        } else {
            // It's ham
            oopspam_store_ham_submission($frmEntry);
            return $spam_level;
        }

    }
    // Return a spam level of 100
    return $spam_level;
};
