<?php
/**
* Plugin Name: OOPSpam Anti-Spam
* Plugin URI: https://www.oopspam.com/
* Description: Stop bots and manual spam from reaching you in comments & contact forms. All with high accuracy, accessibility, and privacy. 
* Version: 1.1.44
* Author: OOPSpam
* URI: https://www.oopspam.com/
* License: GPL2
*/
if (!function_exists('add_action')) {
    die(); 
}
// include a helper class to call the OOPSpam API
require_once(dirname( __FILE__ ).'/OOPSpamAPI.php');

if (is_admin()) { //if admin include the admin specific functions
    require_once(dirname( __FILE__ ).'/options.php');

    	// Loading oopspam plugin review code
	require( 'include/libs/class-plugin-review.php' );
	new OOPSpam_Plugin_Review(
		array(
			'slug'        => 'oopspam', // The plugin slug
			'name'        => 'OOPSpam Anti-Spam', // The plugin name
			'time_limit'  => WEEK_IN_SECONDS,  // The time limit at which notice is shown
		)
	);
}

// Include the plugin helpers.
require_once(dirname( __FILE__ ).'/include/helpers.php');
require_once(dirname( __FILE__ ).'/include/oopspam-country-list.php');
require_once(dirname( __FILE__ ).'/include/oopspam-language-list.php');
require_once(dirname( __FILE__ ).'/include/UI/display-ham-entries.php');
require_once(dirname( __FILE__ ).'/include/UI/display-spam-entries.php');

add_action('init', 'do_output_buffer');
function do_output_buffer() {
        ob_start();
}

// Used to detect installed plugins.
require_once ABSPATH . 'wp-admin/includes/plugin.php';

// Integrations
require_once(dirname( __FILE__ ).'/integration/NinjaForms.php');
require_once(dirname( __FILE__ ).'/integration/GravityForms.php');
require_once(dirname( __FILE__ ).'/integration/ContactForm7.php');
require_once(dirname( __FILE__ ).'/integration/ElementorForm.php');
require_once(dirname( __FILE__ ).'/integration/FluentForms.php');
require_once(dirname( __FILE__ ).'/integration/WPForms.php');
require_once(dirname( __FILE__ ).'/integration/FormidableForms.php');
require_once(dirname( __FILE__ ).'/integration/GiveWP.php');
require_once(dirname( __FILE__ ).'/integration/WPRegistration.php');
require_once(dirname( __FILE__ ).'/integration/BricksForm.php');
require_once(dirname( __FILE__ ).'/integration/WSForm.php');
require_once(dirname( __FILE__ ).'/integration/Toolset.php');
require_once(dirname( __FILE__ ).'/integration/PionetForms.php');
require_once(dirname( __FILE__ ).'/integration/Kadence.php');
require_once(dirname( __FILE__ ).'/integration/WPDiscuz.php');
require_once(dirname( __FILE__ ).'/integration/Mailpoet.php');
require_once(dirname( __FILE__ ).'/integration/Forminator.php');




require_once(dirname( __FILE__ ).'/integration/WooCommerce.php');
add_action( 'plugins_loaded', array( '\OOPSPAM\WOOCOMMERCE\WooSpamProtection', 'getInstance' ) );

require_once(dirname( __FILE__ ).'/db/oopspam-spamentries.php');



register_activation_hook( __FILE__, 'oopspam_plugin_activate' );
register_activation_hook(__FILE__ , 'oopspam_db_install' );
register_deactivation_hook( __FILE__, 'oopspam_plugin_deactivation' );


// Add two weeks & monthly intervals
function oopspam_schedule_intervals($schedules) {

	// add a 'weekly' interval
	$schedules['oopspam-biweekly'] = array(
		'interval' => 1209600,
		'display' => __('Every two weeks')
	);
	$schedules['oopspam-monthly'] = array(
		'interval' => MONTH_IN_SECONDS,
		'display' => __('Once a month')
	);
	return $schedules;
}
add_filter( 'cron_schedules', 'oopspam_schedule_intervals');


    // Schedule Cron Job Event
function oopspam_cron_job() {

    $options = get_option('oopspamantispam_settings');

    // Set default intervals for Ham/Spam Entries table clean up
    if ( ! wp_next_scheduled( 'oopspam_cleanup_ham_entries_cron' ) ) {
        // Once per month 
        wp_schedule_event( strtotime( '+1 month' ), 'oopspam-monthly', 'oopspam_cleanup_ham_entries_cron' );
        $options['oopspam_clear_ham_entries'] = "monthly";
    }
    if ( ! wp_next_scheduled( 'oopspam_cleanup_spam_entries_cron' ) ) {
        // Once per month 
        wp_schedule_event( strtotime( '+1 month' ), 'oopspam-monthly', 'oopspam_cleanup_spam_entries_cron' );
        $options['oopspam_clear_spam_entries'] = "monthly";
    }

    update_option('oopspamantispam_settings', $options );
}


function oopspam_cleanup_ham_entries() {
    // Truncate the table
    $_POST['action_type'] = "empty-entries";
    empty_ham_entries();
}
function oopspam_cleanup_spam_entries() {
    // Truncate the table
    $_POST['action_type'] = "empty-entries";
    empty_spam_entries();
}
add_action( 'oopspam_cleanup_spam_entries_cron', 'oopspam_cleanup_spam_entries' );
add_action( 'oopspam_cleanup_ham_entries_cron', 'oopspam_cleanup_ham_entries' );

function oopspam_plugin_activate() {
    // plugin activated
    do_action( 'oopspam_set_default_settings' );
}

// Set default values
function oopspam_default_options()
{
    $options = get_option('oopspamantispam_settings');
    if (!isset($options['oopspam_api_key_source'])) {
        $default = array(
            'oopspam_is_check_for_length' => true,
            'oopspam_api_key_source' => 'OOPSpamDashboard',
            'oopspam_api_key_usage' => '0/0',
            'oopspam_clear_spam_entries' => 'monthly',
            'oopspam_clear_ham_entries' => 'monthly',
        );
        update_option('oopspamantispam_settings', $default);
    }
}

add_action( 'oopspam_set_default_settings', 'oopspam_default_options' );


function oopspam_plugin_deactivation() {
    // plugin deactivation
    wp_clear_scheduled_hook( 'oopspam_cleanup_ham_entries_cron' );
    wp_clear_scheduled_hook( 'oopspam_cleanup_spam_entries_cron' );
}

add_filter('plugin_action_links', 'oopspam_plugin_action_links', 10, 2);

function oopspam_plugin_action_links($links, $file) {
    static $this_plugin;

    if (!$this_plugin) {
        $this_plugin = plugin_basename(__FILE__);
    }

    if ($file == $this_plugin) {
        // add Settings link on the plugins page
        $settings_link = '<a href="' . get_bloginfo('wpurl') . '/wp-admin/admin.php?page=wp_oopspam_settings_page">Settings</a>';
        array_unshift($links, $settings_link);
    }

    return $links;
}


function oopspamantispam_call_OOPSpam($commentText, $commentIP, $email, $returnReason) {

    $apiKey = oopspamantispam_checkIfValidKey();

    $options = get_option('oopspamantispam_settings');
    $countryallowlistSetting = ( get_option('oopspam_countryallowlist') != null ? get_option('oopspam_countryallowlist') : [""] );
    $languageallowlistSetting = ( get_option('oopspam_languageallowlist') != null ?  get_option('oopspam_languageallowlist') : [""]);
    $checkForLength = ( isset($options['oopspam_is_check_for_length']) ? $options['oopspam_is_check_for_length'] : false ) ;

    // Attempt to anonymize messages
    if (isset($options['oopspam_anonym_content']) && $options['oopspam_anonym_content'] && !empty($commentText)) {
        $email_regex = '/[a-z0-9._%+-]+@[a-z0-9.-]+\.[a-z]{2,}/i';
        $address_regex = '/\d+\s[A-z]+\s[A-z]+/';
        $phoneNumber_regex = '/(?:\+|\d+)(?:\-|\s|\d)/';
        // $name_regex = '/\p{Lu}\p{L}+\s\p{Lu}\p{L}+/';
    
        $commentText = preg_replace($email_regex, '', $commentText);
        $commentText = preg_replace($address_regex, '', $commentText);
        $commentText = preg_replace($phoneNumber_regex, '', $commentText);
        // $commentText = preg_replace($name_regex, '', $commentText);
    }

    // Don't send Email if not allowed by user
    if (isset($options['oopspam_is_check_for_email']) && $options['oopspam_is_check_for_email']) {
        $email = "";
    }

    // Allow devs to apply custom filter and return a score
    // @return $filtered_score: 0 for ham, 6 for spam
    $filtered_score = apply_filters('oopspam_check_for_spam', $commentText, $commentIP, $email);
    if (!empty($filtered_score) && $filtered_score != $commentText) {
        
        $isItHam = $filtered_score < 3 ? true : false ;
        if ($returnReason) {
            $reason = [
                "Score" => $filtered_score,
                "isItHam" => $isItHam
            ];
            return $reason;
        }
        return $isItHam;
    }

    // This is a temporary fix to bypass content length check as GiveWP & Woo usually doesn't have content field
    if (!empty($email) || (empty($email) && empty($commentText))) {
        $checkForLength = false;
    }

    // If length check allowed then anything shorter than 20 should return spam
    if ($checkForLength && strlen($commentText) <= 20) {
        if ($returnReason) {
                $reason = [
                    "Score" => 6,
                    "isItHam" => false
                ];

                return $reason;
            }
        return false;
    }
    
    if(oopspamantispam_checkIfValidKey()) {

        $OOPSpamAPI = new OOPSpamAPI( $apiKey, $checkForLength);

        // Unicode support
        $commentText = mb_convert_encoding($commentText, "UTF-8");

        $response = $OOPSpamAPI->SpamDetection( $commentText, $commentIP, $email, $countryallowlistSetting, $languageallowlistSetting);
        $response_code = wp_remote_retrieve_response_code($response);
        if (!is_wp_error($response) && $response_code == "200") {  
            update_option('over_rate_limit', false );

            $response = json_decode($response['body'], true);

            $currentThreshold = oopspamantispam_get_spamscore_threshold();
            
                if($response['Score'] >= $currentThreshold) { 
                    // It is spam
                    if ($returnReason) {
                        $reason = [
                            "Score" => $response['Score'],
                            "isItHam" => false
                        ];
                        return $reason;
                    }
                    return false;
                } else {
                    // It is ham
                    if ($returnReason) {
                        $reason = [
                            "Score" => $response['Score'],
                            "isItHam" => true
                        ];
                        return $reason;
                    }
                    return true;
                }

        } else if (!is_wp_error($response) && $response_code == "429") {
            // The API limit is reached or some other errors
            update_option('over_rate_limit', true );
        } else {
            // Allow all submission as no analyses are done.
            return $returnReason ? ["Score" => 0, "isItHam" => true] : true;
            if(is_wp_error($response)) {
                echo $response->get_error_message();
            }
        }
        unset($OOPSpamAPI);
    }
    // return $returnReason ? ["isItHam" => true] : true;
}

function oopspamantispam_report_OOPSpam($commentText, $commentIP, $email, $isSpam) {

    $apiKey = oopspamantispam_checkIfValidKey();

    $options = get_option('oopspamantispam_settings');
    $countryallowlistSetting = ( get_option('oopspam_countryallowlist') != null ? get_option('oopspam_countryallowlist') : [""] );
    $languageallowlistSetting = ( get_option('oopspam_languageallowlist') != null ?  get_option('oopspam_languageallowlist') : [""]);
    $checkForLength = ( isset($options['oopspam_is_check_for_length']) ? $options['oopspam_is_check_for_length'] : false ) ;

    if(oopspamantispam_checkIfValidKey()) {

        $OOPSpamAPI = new OOPSpamAPI( $apiKey, $checkForLength);
        $response = $OOPSpamAPI->Report( $commentText, $commentIP, $email, $countryallowlistSetting, $languageallowlistSetting, $isSpam);

        $response_code = wp_remote_retrieve_response_code($response);
        if (!is_wp_error($response) && $response_code == "201") {  
            $response = json_decode($response['body'], true);
            return $response['message'];
        } else {
            if(is_wp_error($response)) {
                echo $response->get_error_message();
            }
            return false;
        }
        unset($OOPSpamAPI);
    }
}

function oopspamantispam_get_ip_address() {
    return isset( $_SERVER['REMOTE_ADDR'] ) ? $_SERVER['REMOTE_ADDR'] : '::1';
}

// Remove http & https from domain
function urlToDomain($url) {
    return implode(array_slice(explode('/', preg_replace('/https?:\/\/(www\.)?/', '', $url)), 0, 1));
 }

function oopspamantispam_check_comment($comment_ID, $comment_approved, $commentdata) {
    $senderIp = ""; $email = ""; $isItSpam = false;
    $options = get_option('oopspamantispam_settings');
    $currentSpamFolder = oopspamantispam_get_folder_for_spam();

    $checkForLength = ( isset($options['oopspam_is_check_for_length']) ? $options['oopspam_is_check_for_length'] : false ) ;


    if (!isset($options['oopspam_is_check_for_ip']) || $options['oopspam_is_check_for_ip'] != true) {
        $senderIp = oopspamantispam_get_ip_address();
    }

    if (!isset($options['oopspam_is_check_for_email']) || $options['oopspam_is_check_for_email'] != true) {
        $email = sanitize_email($commentdata['comment_author_email']);
    }

    $trimmedURL = urlToDomain($commentdata['comment_author_url']);

    // Capture non-URLs that doesn't contain dot and able to bypass WP's validation
    if (!empty($trimmedURL) && strpos($trimmedURL, ".") === false) {
        $isItSpam = true;
    }

    $content = $commentdata['comment_content']." ".$commentdata['comment_author_url'];

    // If length check allowed then anything shorter than 20 should be considered as spam
    if ($checkForLength && strlen($commentdata['comment_content']) <= 20) {
        $isItSpam = true;
    } else if (oopspamantispam_call_OOPSpam(sanitize_textarea_field($content), $senderIp, $email, false) == false) {
        // if Spam filtering is on and the OOPSpam Service considers it spam then mark it as spam
        $isItSpam = true;
    }    
    
    // Move the spam comment select folder (Trash or spam) 
    if ($isItSpam) {
        $currentSpamFolder === "trash" ? wp_trash_comment( $comment_ID ) : wp_spam_comment( $comment_ID );
        // TODO: Allow users to customize this message
        wp_die( __( 'Your comment has been flagged as spam.', 'oopspam' ) );
    }
    return $commentdata;
}

function oopspamantispam_result_pending() {
    return 0;
}

add_action( 'comment_post' , 'oopspamantispam_check_comment', 10, 3 ); 

add_action('admin_init', 'oopspam_admin_init');

// add_action( 'admin_init', 'oopspam_cron_job' );

// load the main.css style
function oopspam_admin_init() {
   
    // Add corn jobs
    oopspam_cron_job();

    wp_register_style('oopspam_stylesheet', plugins_url ( 'styles/main.css', __FILE__ ));
    wp_register_style('chosen', plugins_url ( './include/libs/chosen.css', __FILE__ ));
    add_action('admin_print_styles' ,'oopspam_admin_style');
   
    wp_enqueue_script( 'chosen', plugins_url( './include/libs/chosen.jquery.js', __FILE__ ), array( 'jquery' ), false, true);
    wp_enqueue_script( 'helper', plugins_url( './include/helper.js', __FILE__ ), array( 'jquery', 'chosen' ), false, true);
    
function oopspam_admin_style() {
    wp_enqueue_style('oopspam_stylesheet');
    wp_enqueue_style('chosen');
    }
}